#!/usr/bin/python
## Copyright (C) 2003 Red Hat, Inc.
## Copyright (C) 2003 Dan Walsh <dwalsh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import os.path
import gettext
import urllib
import glob
from netboot_util import *
from xml.dom import minidom

OSList=[]
def firsttime():
    pxelist=pxeOSList()
    return pxelist.isFirstTime()

class pxeOSList:
    def __init__(self):
        self.configFile=PXECFGDIR + "pxeos.xml"
        try:
            self.dom = minidom.parse(self.configFile)
            self.firsttime=FALSE
        except IOError,msg:
            self.dom = minidom.parseString("<OperatingSystems/>")
            self.firsttime=TRUE

    def getOS(self,name):
        for OS in self.getOSList():
            if OS.getName() == name:
                return OS
        raise ValueError,_("%s does not exist") % name
        
    def delete(self,name):
        self.getOS(name) # check to make sure name exists
        os_elements = self.dom.getElementsByTagName("OS")
        for m in os_elements:
            if m.getAttribute("Name") == name:
                m.parentNode.removeChild(m)
                break
        self.save()

    def isFirstTime(self):
        return self.firsttime

    def getOSList(self):
        OSList=[]
        os_elements = self.dom.getElementsByTagName("OS")
        for m in os_elements:
            OS=pxeos()
            OS.setName(m.getAttribute("Name"))
            OS.setProtocol(m.getAttribute("Protocol"))
            OS.setDiskless(m.getAttribute("Diskless")=="1")
            OS.setDescription(m.getAttribute("Description"))
            OS.setAnonymous(m.getAttribute("Anonymous")=="1")
            OS.setLocation(m.getAttribute("Location"))
            OS.setServer(m.getAttribute("Server"))
            OS.setUser(m.getAttribute("User"))
            OS.setPassword(m.getAttribute("Password"))
            OS.setKernel(m.getAttribute("Kernel"))
            OSList.append(OS)

        return OSList

    def append(self,OS):
        osList=self.getOSList()
        osList.append(OS)
        self.saveOSList(osList)
        
    def saveOSList(self,osList):
        self.dom = minidom.parseString("<OperatingSystems/>")
        OSList=osList

        inputBootFD=open("%s/msgs/boot.msg" % INSTALLDIR,"r")
        bootFD=open("%s/msgs/boot.msg" % PXEBOOTDIR,"w")
        input=inputBootFD.read()
        inputBootFD.close()
        bootFD.write(input)

        inputDefaultFD=open("%s/pxelinux.cfg/default" % INSTALLDIR,"r")
        defaultFD=open("%s/pxelinux.cfg/default" % PXEBOOTDIR,"w")
        input=inputDefaultFD.read()
        inputDefaultFD.close()
        defaultFD.write(input)
        ctr = 1

        for i in OSList:
            osElement = self.dom.getElementsByTagName("OperatingSystems")[0]
            spacer=self.dom.createTextNode("\n     ")
            element = self.dom.createElement("OS")
            new_element=osElement.appendChild(element)
            osElement.insertBefore(spacer, new_element)
            new_element.setAttribute("Name", i.getName()) 
            new_element.setAttribute("Protocol", i.getProtocol())
            new_element.setAttribute("Diskless", str(i.isDiskless()))
            new_element.setAttribute("Description", i.getDescription())
            new_element.setAttribute("Anonymous", str(i.isAnonymous()))
            new_element.setAttribute("Server", i.getServer())
            new_element.setAttribute("Location", i.getLocation())
            new_element.setAttribute("User", i.getUser()) 
            new_element.setAttribute("Password", i.getPassword()) 
            new_element.setAttribute("Kernel", i.getKernel()) 

            if not i.isDiskless():
                bootFD.write("%d. %s - %s \n" % (ctr,i.getName(),i.getDescription()))
                defaultFD.write("\n\nlabel %d\n" % ctr)
                defaultFD.write("  kernel \"%s\"/vmlinuz\n" % i.getNameDir())
                defaultFD.write("  append initrd=\"%s\"/initrd.img ramdisk_size=10000\n" % (i.getNameDir()))
                ctr = ctr + 1
            
        if len(osList):
            nl=self.dom.createTextNode("\n ")
            osElement.appendChild(nl)
        self.save()
        
        bootFD.write("\n\n\n05[F1-Main] [F2-General] [F3-Expert] [F4-Kernel] [F5-Rescue]07\n")
        bootFD.close()
        defaultFD.close()
        
    def save(self):
        cFD=open(self.configFile,"w")
        cFD.write(self.dom.toxml())
        cFD.close()
        
    def out(self):
        print self.dom.toxml()

class pxeos:
    def __init__(self,name=""):
        self.setProtocol("HTTP")
        self.setName(name)
        self.setDescription("")
        self.setAnonymous(TRUE)
        self.setLocation("")
        self.setServer(hostname())
        self.setUser("")
        self.setPassword("")
        self.setDiskless(FALSE)
        self.setKernel("")
        self.tmpdir = None

    def mount(self):
        self.unmount()
        rootDir=self.location + "/root"
        self.tmpdir="/tmp"+rootDir
        mkdir(self.tmpdir)
        rc = getstatusoutput("/bin/mount %s:\"%s\" \"%s\"" % (self.server,rootDir,self.tmpdir))

    def getKernels(self):
        return glob.glob(self.tmpdir+"/boot/vmlinuz-*")
    
    def unmount(self):
        if self.tmpdir != None:
            getstatusoutput("/bin/umount %s" % self.tmpdir)
            self.tmpdir = None

    def makeDiskless(self):
        if self.isDiskless():
            rc=getstatusoutput("/usr/share/%s/diskless/mkdiskless %s" % (PROGNAME,self.location))
            return rc[0]
        return 0
    
    def __getRemoteFile(self,file):
        if self.protocol=="FTP" and not self.isAnonymous():
            server="%s:%s@%s" % (self.user,self.password,self.server)
        else:
            server=self.server
        outlocation=self.getPXEDir()
        if self.protocol != "NFS":
            urlopener=urllib.URLopener()
            try:
                if server != "":
                    top="%s://%s/%s/images/pxeboot" % (self.protocol,self.server,self.location)
                else:
                    top="%s://%s/images/pxeboot" % (self.protocol,self.location)
                f="%s/%s" % (top,file)
                infile = urlopener.open(f)
            except IOError:
                if self.location != "":
                    top="%s://%s/%s/disc1/images/pxeboot" % (self.protocol,server,self.location)
                else:
                    top="%s://%s/disc1/images/pxeboot" % (self.protocol,server)
                f="%s/%s" % (top,file)
                infile = urlopener.open(f)
            mkdir(outlocation)
            fd = open("%s/%s" % (outlocation, file),"w")
            fd.write(infile.read())
            infile.close()
            if fd.tell()==0:
                raise IOError,"File Not Found"
            fd.close()
        else:
            localmnt="/mnt/%s" % self.name
            mkdir(localmnt)
            rc = getstatusoutput("/bin/mount %s:\"%s\" \"%s\"" % (self.server,self.location,localmnt))
            if rc[0] != 0:
                raise IOError,"Error Mounting: %s" % rc[1]
                
            try:
                f="%s/disc1/images/pxeboot/%s" % (localmnt,file)
                infile = open(f,"r")
            except IOError:
                f="%s/images/pxeboot/%s" % (localmnt,file)
                infile = open(f,"r")

            mkdir(outlocation)
            fd = open("%s/%s" % (outlocation, file),"w")
            fd.write(infile.read())
            infile.close()
            rc = getstatusoutput("/bin/umount \"%s\"" % (localmnt))
            os.rmdir(localmnt)
            if fd.tell()==0:
                raise IOError,_("Error Copying: $s File Not Found") % file
            fd.close()

    def __setupKSFile(self):
        filename = "%s/ks.cfg" % self.getPXEDir()
        location = self.getLocation()
        server = self.getServer()
        ks=open(filename,"w")
        ks.write("#use %s installation media\n" % self.protocol)
        if self.protocol=="NFS":
            ks.write("nfs --server %s --dir %s\n" % (server,location))
        if self.protocol=="HTTP":
            ks.write("url --url http://%s/%s\n" % (server,location))
        if self.protocol=="FTP":
            if self.getUser()!="":
                ks.write("url --url ftp://%s:%s@%s/%s\n" % (server,self.getUser(),self.getPassword(),location))
            else:
                ks.write("url --url ftp://%s/%s\n" % (server,location))
        ks.close()
        
    def updatePXE(self):
        rc=(0,"")
        if self.isDiskless():
            rc=getstatusoutput("/usr/share/%s/diskless/updateDiskless \"%s\" \"%s\" \"%s\"" % (PROGNAME,self.tmpdir,self.kernel,self.getPXEDir()))
            self.unmount()
        else:
            self.__getRemoteFile("vmlinuz")
            self.__getRemoteFile("initrd.img")
            self.__setupKSFile()
        return rc

    def setName(self,name):
        self.name=name
        
    def getName(self):
        return self.name
        
    def setDescription(self,description):
        self.description=description
        
    def getDescription(self):
        return self.description
        
    def getNameDir(self):
        return self.name.replace(" ","_")
     
    def setDiskless(self,diskless):
        if diskless and self.getProtocol() != "NFS":
           raise ValueError,_("Diskless requires NFS protocol")
        self.diskless=diskless
        
    def isDiskless(self):
        return self.diskless
        
    def setProtocol(self,protocol):
        PROTOCOLS=["HTTP","NFS","FTP"]
        if protocol in PROTOCOLS:
            self.protocol=protocol
        else:
            raise ValueError,_("%s is not a valid protocol") % protocol
        
        
    def getProtocol(self):
        return self.protocol
        
    def setLocation(self,loc):
        location=os.path.realpath(loc)
        b=os.path.basename(location)
        if b=="root":
            self.location=os.path.dirname(location)
        else:
            self.location=location
        
    def getLocation(self):
        return self.location
        
    def setServer(self,server):
        self.server=server
        
    def getServer(self):
        return self.server
        
    def setUser(self,user):
        self.user=user
        
    def getUser(self):
        return self.user

    def setPassword(self,password):
        self.password=password
        
    def getPassword(self):
        return self.password

    def setAnonymous(self,anonymous):
        self.anonymous=anonymous
        
    def isAnonymous(self):
        return self.anonymous

    def getKernel(self):
        return self.kernel;
        
    def getKernelString(self):
        return "vmlinuz-%s" % self.kernel;
        
    def setKernel(self,ver):
        self.kernel=ver;

    def getPXEDir(self):
        return "/tftpboot/linux-install/" + self.getNameDir()

    def out(self):
        print self.getName()
        print "\tDescription:\t%s" % self.getDescription()
        print "\tProtocol:\t%s" % self.getProtocol()
        print "\tisDiskless:\t%s" % self.isDiskless()
        print "\tServer:\t\t%s" % self.getServer()
        print "\tLocation:\t%s" % self.getLocation()
        if self.getProtocol() == "FTP":
            print "\tisAnonymous:\t%s" % self.isAnonymous()
            print "\tUser:\t%s" % self.getUser()
            print "\tPassword:\t%s" % self.getPassword()

def Usage():
    print _("pxeos - Python pxeos commandline tool\n\nUsage: pxeos [-a|-m|-d|-l] [-h] [-i <description> ] [-p <protocol> ] [-D<1|0>] [-s <server>] [-L <location>] [-A <1|0>] [-u <user>] [-P <password>] name")

if __name__ == "__main__":
    
    import sys
    import signal
    import getopt

    if os.getuid() != 0:
        print _("Please restart %s with root permissions!") % (sys.argv[0])
        sys.exit(10)
        
    if len(sys.argv) == 1:
        Usage()
        sys.exit(1)
        
    cmdline = sys.argv[1:]
    sys.argv = sys.argv[:1]

    signal.signal (signal.SIGINT, signal.SIG_DFL)
    class BadUsage: pass
    
    name = cmdline[-1]
    kernel=None
    description=None
    protocol=None
    diskless=None
    anonymous=None
    server=None
    location=None
    user=None
    password=None

    add=FALSE
    modify=FALSE
    delete=FALSE
    progname = os.path.basename(sys.argv[0])

    try:
        opts, args = getopt.getopt(cmdline, "amdi:k:p:D:s:L:A:u:P:lh", ["add","modify","delete", "description=","kernel=","protocol=","diskless=","server=","location=","anonymous=","user=","password=","list","help"])
        for opt, val in opts:
            
            if opt == '-a' or opt == '--add':
                add=TRUE

            if opt == '-m' or opt == '--modify':
                modify=TRUE

            if opt == '-d' or opt == '--delete':
                delete=TRUE

            if opt == '-i' or opt == '--description':
                description=val

            if opt == '-p' or opt == '--protocol':
                protocol=val

            if opt == '-D' or opt == '--diskless':
                diskless=val

            if opt == '-k' or opt == '--kernel':
                kernel=val

            if opt == '-A' or opt == '--anonymous':
                anonymous=val

            if opt == '-s' or opt == '--server':
                server=val

            if opt == '-L' or opt == '--location':
                location=val

            if opt == '-u' or opt == '--user':
                user=val

            if opt == '-P' or opt == '--password':
                password=val

            if opt == '-l' or opt == '--list':
                for pxeos in pxeOSList().getOSList():
                    pxeos.out()
                sys.exit(0)

            if opt == '-h' or opt == '--help':
                Usage()
                sys.exit(0)

    except (getopt.error, BadUsage):
        Usage()
        sys.exit(1)

    if (add and delete) or (add and modify) or (modify and delete):
        Usage()
        sys.exit(1)

    try:
        pxeoslist=pxeOSList()
        if delete:
            pxeoslist.delete(name)
            sys.exit(0)

        pxelist=pxeoslist.getOSList()
        if add or modify:
            if add:
                for i in pxelist:
                    if i.getName()==name:
                        raise ValueError,_("%s already exists") % name
                pxe=pxeos(name)
                pxelist.append(pxe)
            if modify:
                found=FALSE
                for pxe in pxelist:
                    if pxe.getName()==name:
                        found=TRUE
                        break
                if not found:
                    raise ValueError,_("%s does not exist") % name

            if description != None:
                pxe.setDescription(description)
            if protocol != None:
                pxe.setProtocol(protocol)
            if diskless != None:
                pxe.setDiskless(diskless=="1")
            if server != None:
                pxe.setServer(server)
            if kernel != None:
                pxe.setKernel(kernel)
            if location != None:
                pxe.setLocation(location)
            if anonymous != None:
                pxe.setAnonymous(anonymous=="1")
            if user != None:
                pxe.setUser(user)
            if password != None:
                pxe.setPassword(password)
            
            if pxe.isDiskless():
                pxe.mount()
                pxe.makeDiskless()

            try:
                pxe.updatePXE()
            except IOError,e:
                print(_("You must enter directory of the Operating System Software\nThis directory must include the images/pxeboot directories\n%s" % e))
                pxe.unmount()
                sys.exit(1)
            pxeoslist.saveOSList(pxelist)
            pxe.unmount()
            sys.exit(0)

        found=FALSE
        for pxe in pxelist:
            if pxe.getName()==name:
                found=TRUE
                break
        if not found:
            raise ValueError,_("%s does not exist") % name
        pxe.out()
        
    except ValueError,e:
        print _("Error: %s") % e
        sys.exit(1)
        

